@@ -2,7 +2,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2014 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2015-2016 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2015-2018 Takao Fujiwara <takao.fujiwara1@gmail.com>
  * Copyright (C) 2008-2016 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -127,6 +127,7 @@ enum {
     ENGINE_CHANGED,
     REQUEST_ENGINE,
     SET_CONTENT_TYPE,
+    PANEL_EXTENSION,
     LAST_SIGNAL,
 };
 
@@ -161,9 +162,6 @@ static gboolean bus_input_context_service_set_property
                                     GError               **error);
 static void     bus_input_context_unset_engine
                                    (BusInputContext       *context);
-static void     bus_input_context_commit_text
-                                   (BusInputContext       *context,
-                                    IBusText              *text);
 static void     bus_input_context_update_preedit_text
                                    (BusInputContext       *context,
                                     IBusText              *text,
@@ -601,6 +599,17 @@ bus_input_context_class_init (BusInputContextClass *class)
             G_TYPE_UINT,
             G_TYPE_UINT);
 
+    context_signals[PANEL_EXTENSION] =
+        g_signal_new (I_("panel-extension"),
+            G_TYPE_FROM_CLASS (class),
+            G_SIGNAL_RUN_LAST,
+            0,
+            NULL, NULL,
+            bus_marshal_VOID__VARIANT,
+            G_TYPE_NONE,
+            1,
+            G_TYPE_VARIANT);
+
     text_empty = ibus_text_new_from_string ("");
     g_object_ref_sink (text_empty);
     lookup_table_empty = ibus_lookup_table_new (9 /* page size */, 0, FALSE, FALSE);
@@ -673,8 +682,10 @@ bus_input_context_send_signal (BusInputContext *context,
                                GVariant        *parameters,
                                GError         **error)
 {
-    if (context->connection == NULL)
+    if (context->connection == NULL) {
+        g_variant_unref (parameters);
         return TRUE;
+    }
 
     GDBusMessage *message = g_dbus_message_new_signal (ibus_service_get_object_path ((IBusService *)context),
                                                        interface_name,
@@ -704,8 +715,10 @@ bus_input_context_emit_signal (BusInputContext *context,
                                GVariant        *parameters,
                                GError         **error)
 {
-    if (context->connection == NULL)
+    if (context->connection == NULL) {
+        g_variant_unref (parameters);
         return TRUE;
+    }
 
     return bus_input_context_send_signal (context,
                                           "org.freedesktop.IBus.InputContext",
@@ -1147,6 +1160,20 @@ _ic_set_surrounding_text (BusInputContext       *context,
     g_dbus_method_invocation_return_value (invocation, NULL);
 }
 
+/*
+ * Since IBusService is inherited by IBusImpl, this method cannot be
+ * applied to IBusServiceClass.method_call() directly but can be in
+ * each child class.method_call().
+ */
+static gboolean
+bus_input_context_service_authorized_method (IBusService     *service,
+                                             GDBusConnection *connection)
+{
+    if (ibus_service_get_connection (service) == connection)
+        return TRUE;
+    return FALSE;
+}
+
 /**
  * bus_input_context_service_method_call:
  *
@@ -1196,6 +1223,10 @@ bus_input_context_service_method_call (IBusService            *service,
     };
 
     gint i;
+
+    if (!bus_input_context_service_authorized_method (service, connection))
+        return;
+
     for (i = 0; i < G_N_ELEMENTS (methods); i++) {
         if (g_strcmp0 (method_name, methods[i].method_name) == 0) {
             methods[i].method_callback ((BusInputContext *)service, parameters, invocation);
@@ -1269,6 +1300,9 @@ bus_input_context_service_set_property (IBusService     *service,
                                   error);
     }
 
+    if (!bus_input_context_service_authorized_method (service, connection))
+        return FALSE;
+
     if (g_strcmp0 (property_name, "ContentType") == 0) {
         BusInputContext *context = (BusInputContext *) service;
         _ic_set_content_type (context, value);
@@ -1278,6 +1312,7 @@ bus_input_context_service_set_property (IBusService     *service,
     g_return_val_if_reached (FALSE);
 }
 
+
 gboolean
 bus_input_context_has_focus (BusInputContext *context)
 {
@@ -1430,22 +1465,6 @@ bus_input_context_property_activate (BusInputContext *context,
     }
 }
 
-static void
-bus_input_context_commit_text (BusInputContext *context,
-                               IBusText        *text)
-{
-    g_assert (BUS_IS_INPUT_CONTEXT (context));
-
-    if (text == text_empty || text == NULL)
-        return;
-
-    GVariant *variant = ibus_serializable_serialize ((IBusSerializable *)text);
-    bus_input_context_emit_signal (context,
-                                   "CommitText",
-                                   g_variant_new ("(v)", variant),
-                                   NULL);
-}
-
 /**
  * bus_input_context_update_preedit_text:
  *
@@ -2095,6 +2114,20 @@ _engine_update_property_cb (BusEngineProxy  *engine,
     bus_input_context_update_property (context, prop);
 }
 
+/**
+ * _engine_panel_extension_cb:
+ *
+ * A function to be called when "panel-extension" glib signal is sent
+ * from the engine object.
+ */
+static void
+_engine_panel_extension_cb (BusEngineProxy  *engine,
+                            GVariant        *parameters,
+                            BusInputContext *context)
+{
+    g_signal_emit (context, context_signals[PANEL_EXTENSION], 0, parameters);
+}
+
 #define DEFINE_FUNCTION(name)                                   \
     static void                                                 \
     _engine_##name##_cb (BusEngineProxy   *engine,              \
@@ -2237,7 +2270,8 @@ const static struct {
     { "cursor-down-lookup-table", G_CALLBACK (_engine_cursor_down_lookup_table_cb) },
     { "register-properties",      G_CALLBACK (_engine_register_properties_cb) },
     { "update-property",          G_CALLBACK (_engine_update_property_cb) },
-    { "destroy",                  G_CALLBACK (_engine_destroy_cb) },
+    { "panel-extension",          G_CALLBACK (_engine_panel_extension_cb) },
+    { "destroy",                  G_CALLBACK (_engine_destroy_cb) }
 };
 
 static void
@@ -2601,3 +2635,19 @@ bus_input_context_set_content_type (BusInputContext *context,
     _ic_set_content_type (context, value);
     g_variant_unref (value);
 }
+
+void
+bus_input_context_commit_text (BusInputContext *context,
+                               IBusText        *text)
+{
+    g_assert (BUS_IS_INPUT_CONTEXT (context));
+
+    if (text == text_empty || text == NULL)
+        return;
+
+    GVariant *variant = ibus_serializable_serialize ((IBusSerializable *)text);
+    bus_input_context_emit_signal (context,
+                                   "CommitText",
+                                   g_variant_new ("(v)", variant),
+                                   NULL);
+}
