@@ -2,7 +2,8 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2013 Red Hat, Inc.
+ * Copyright (C) 2013-2018 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2008-2018 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -42,9 +43,15 @@ static gboolean xim = FALSE;
 static gboolean replace = FALSE;
 static gboolean restart = FALSE;
 static gchar *panel = "default";
+static gchar *panel_extension = "default";
 static gchar *config = "default";
 static gchar *desktop = "gnome";
 
+static gchar *panel_extension_disable_users[] = {
+    "gdm",
+    "liveuser"
+};
+
 static void
 show_version_and_quit (void)
 {
@@ -60,11 +67,12 @@ static const GOptionEntry entries[] =
     { "xim",       'x', 0, G_OPTION_ARG_NONE,   &xim,       "execute ibus XIM server.", NULL },
     { "desktop",   'n', 0, G_OPTION_ARG_STRING, &desktop,   "specify the name of desktop session. [default=gnome]", "name" },
     { "panel",     'p', 0, G_OPTION_ARG_STRING, &panel,     "specify the cmdline of panel program. pass 'disable' not to start a panel program.", "cmdline" },
+    { "panel-extension", 'E', 0, G_OPTION_ARG_STRING, &panel_extension, "specify the cmdline of panel extension program. pass 'disable' not to start an extension program.", "cmdline" },
     { "config",    'c', 0, G_OPTION_ARG_STRING, &config,    "specify the cmdline of config program. pass 'disable' not to start a config program.", "cmdline" },
     { "address",   'a', 0, G_OPTION_ARG_STRING, &g_address,   "specify the address of ibus daemon.", "address" },
     { "replace",   'r', 0, G_OPTION_ARG_NONE,   &replace,   "if there is an old ibus-daemon is running, it will be replaced.", NULL },
     { "cache",     't', 0, G_OPTION_ARG_STRING, &g_cache,   "specify the cache mode. [auto/refresh/none]", NULL },
-    { "timeout",   'o', 0, G_OPTION_ARG_INT,    &g_gdbus_timeout, "gdbus reply timeout in milliseconds. pass -1 to use the default timeout of gdbus.", "timeout [default is 5000]" },
+    { "timeout",   'o', 0, G_OPTION_ARG_INT,    &g_gdbus_timeout, "gdbus reply timeout in milliseconds. pass -1 to use the default timeout of gdbus.", "timeout [default is 15000]" },
     { "mem-profile", 'm', 0, G_OPTION_ARG_NONE,   &g_mempro,   "enable memory profile, send SIGUSR2 to print out the memory profile.", NULL },
     { "restart",     'R', 0, G_OPTION_ARG_NONE,   &restart,    "restart panel and config processes when they die.", NULL },
     { "verbose",   'v', 0, G_OPTION_ARG_NONE,   &g_verbose,   "verbose.", NULL },
@@ -167,6 +175,9 @@ _sig_usr2_handler (int sig)
 gint
 main (gint argc, gchar **argv)
 {
+    int i;
+    const gchar *username = ibus_get_user_name ();
+
     setlocale (LC_ALL, "");
 
     GOptionContext *context = g_option_context_new ("- ibus daemon");
@@ -191,9 +202,7 @@ main (gint argc, gchar **argv)
 
     /* check uid */
     {
-        const gchar *username = ibus_get_user_name ();
-        uid_t uid = getuid ();
-        struct passwd *pwd = getpwuid (uid);
+        struct passwd *pwd = getpwuid (getuid ());
 
         if (pwd == NULL || g_strcmp0 (pwd->pw_name, username) != 0) {
             g_printerr ("Please run ibus-daemon with login user! Do not run ibus-daemon with sudo or su.\n");
@@ -234,6 +243,12 @@ main (gint argc, gchar **argv)
     }
 
     bus_server_init ();
+    for (i = 0; i < G_N_ELEMENTS(panel_extension_disable_users); i++) {
+        if (!g_strcmp0 (username, panel_extension_disable_users[i]) != 0) {
+            panel_extension = "disable";
+            break;
+        }
+    }
     if (!single) {
         /* execute config component */
         if (g_strcmp0 (config, "default") == 0) {
@@ -270,6 +285,26 @@ main (gint argc, gchar **argv)
         }
     }
 
+#ifdef EMOJI_DICT
+    if (g_strcmp0 (panel_extension, "default") == 0) {
+        BusComponent *component;
+        component = bus_ibus_impl_lookup_component_by_name (
+                BUS_DEFAULT_IBUS, IBUS_SERVICE_PANEL_EXTENSION);
+        if (component) {
+            bus_component_set_restart (component, restart);
+        }
+        if (component == NULL ||
+            !bus_component_start (component, g_verbose)) {
+            g_printerr ("Can not execute default panel program\n");
+            exit (-1);
+        }
+    } else if (g_strcmp0 (panel_extension, "disable") != 0 &&
+               g_strcmp0 (panel_extension, "") != 0) {
+        if (!execute_cmdline (panel_extension))
+            exit (-1);
+    }
+#endif
+
     /* execute ibus xim server */
     if (xim) {
         if (!execute_cmdline (LIBEXECDIR "/ibus-x11 --kill-daemon"))
