@@ -2,7 +2,8 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2014 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2014 Red Hat, Inc.
+ * Copyright (C) 2017-2018 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2008-2018 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -49,6 +50,8 @@ enum {
     PROPERTY_ACTIVATE,
     PROPERTY_SHOW,
     PROPERTY_HIDE,
+    COMMIT_TEXT,
+    PANEL_EXTENSION,
     LAST_SIGNAL,
 };
 
@@ -57,6 +60,7 @@ struct _BusPanelProxy {
 
     /* instance members */
     BusInputContext *focused_context;
+    PanelType panel_type;
 };
 
 struct _BusPanelProxyClass {
@@ -75,6 +79,8 @@ struct _BusPanelProxyClass {
     void (* property_activate)  (BusPanelProxy   *panel,
                                  const gchar     *prop_name,
                                  gint             prop_state);
+    void (* commit_text)        (BusPanelProxy   *panel,
+                                 IBusText        *text);
 };
 
 static guint    panel_signals[LAST_SIGNAL] = { 0 };
@@ -99,26 +105,46 @@ static void     bus_panel_proxy_property_activate
                                                 (BusPanelProxy          *panel,
                                                  const gchar            *prop_name,
                                                  gint                    prop_state);
+static void     bus_panel_proxy_commit_text
+                                                (BusPanelProxy          *panel,
+                                                 IBusText               *text);
 
 G_DEFINE_TYPE(BusPanelProxy, bus_panel_proxy, IBUS_TYPE_PROXY)
 
 BusPanelProxy *
-bus_panel_proxy_new (BusConnection *connection)
+bus_panel_proxy_new (BusConnection *connection,
+                     PanelType      panel_type)
 {
+    const gchar *path = NULL;
+    GObject *obj;
+    BusPanelProxy *panel;
+
     g_assert (BUS_IS_CONNECTION (connection));
 
-    GObject *obj;
+    switch (panel_type) {
+    case PANEL_TYPE_PANEL:
+        path = IBUS_PATH_PANEL;
+        break;
+    case PANEL_TYPE_EXTENSION:
+        path = IBUS_PATH_PANEL_EXTENSION;
+        break;
+    default:
+        g_return_val_if_reached (NULL);
+    }
+
     obj = g_initable_new (BUS_TYPE_PANEL_PROXY,
                           NULL,
                           NULL,
-                          "g-object-path",     IBUS_PATH_PANEL,
+                          "g-object-path",     path,
                           "g-interface-name",  IBUS_INTERFACE_PANEL,
                           "g-connection",      bus_connection_get_dbus_connection (connection),
                           "g-default-timeout", g_gdbus_timeout,
                           "g-flags",           G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START | G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
                           NULL);
 
-    return BUS_PANEL_PROXY (obj);
+    panel = BUS_PANEL_PROXY (obj);
+    panel->panel_type = panel_type;
+    return panel;
 }
 
 static void
@@ -133,6 +159,7 @@ bus_panel_proxy_class_init (BusPanelProxyClass *class)
     class->cursor_down = bus_panel_proxy_cursor_down;
     class->candidate_clicked = bus_panel_proxy_candidate_clicked;
     class->property_activate = bus_panel_proxy_property_activate;
+    class->commit_text = bus_panel_proxy_commit_text;
 
     /* install signals */
     panel_signals[PAGE_UP] =
@@ -214,6 +241,32 @@ bus_panel_proxy_class_init (BusPanelProxyClass *class)
             G_TYPE_NONE, 1,
             G_TYPE_STRING);
 
+    panel_signals[COMMIT_TEXT] =
+        g_signal_new (I_("commit-text"),
+            G_TYPE_FROM_CLASS (class),
+            G_SIGNAL_RUN_LAST,
+            G_STRUCT_OFFSET(BusPanelProxyClass, commit_text),
+            NULL, NULL,
+            bus_marshal_VOID__OBJECT,
+            G_TYPE_NONE, 1,
+            IBUS_TYPE_TEXT);
+
+    panel_signals[PANEL_EXTENSION] =
+        g_signal_new (I_("panel-extension"),
+            G_TYPE_FROM_CLASS (class),
+            G_SIGNAL_RUN_LAST,
+            0,
+            NULL, NULL,
+            bus_marshal_VOID__VARIANT,
+            G_TYPE_NONE, 1,
+            G_TYPE_VARIANT);
+}
+
+static void
+_g_object_unref_if_floating (gpointer instance)
+{
+    if (g_object_is_floating (instance))
+        g_object_unref (instance);
 }
 
 static void
@@ -232,7 +285,8 @@ bus_panel_proxy_real_destroy (IBusProxy *proxy)
         panel->focused_context = NULL;
     }
 
-    IBUS_PROXY_CLASS(bus_panel_proxy_parent_class)->destroy ((IBusProxy *)panel);
+    IBUS_PROXY_CLASS(bus_panel_proxy_parent_class)->
+            destroy ((IBusProxy *)panel);
 }
 
 /**
@@ -299,6 +353,28 @@ bus_panel_proxy_g_signal (GDBusProxy  *proxy,
         return;
     }
 
+    if (g_strcmp0 ("CommitText", signal_name) == 0) {
+        GVariant *arg0 = NULL;
+        g_variant_get (parameters, "(v)", &arg0);
+        g_return_if_fail (arg0 != NULL);
+
+        IBusText *text = IBUS_TEXT (ibus_serializable_deserialize (arg0));
+        g_variant_unref (arg0);
+        g_return_if_fail (text != NULL);
+        g_signal_emit (panel, panel_signals[COMMIT_TEXT], 0, text);
+        _g_object_unref_if_floating (text);
+        return;
+    }
+
+    if (g_strcmp0 ("PanelExtension", signal_name) == 0) {
+        if (panel->panel_type != PANEL_TYPE_PANEL) {
+            g_warning ("Wrong signal");
+            return;
+        }
+        g_signal_emit (panel, panel_signals[PANEL_EXTENSION], 0, parameters);
+        return;
+    }
+
     /* shound not be reached */
     g_return_if_reached ();
 }
@@ -472,6 +548,18 @@ bus_panel_proxy_property_activate (BusPanelProxy *panel,
     }
 }
 
+static void
+bus_panel_proxy_commit_text (BusPanelProxy *panel,
+                             IBusText      *text)
+{
+    g_assert (BUS_IS_PANEL_PROXY (panel));
+    g_assert (text != NULL);
+
+    if (panel->focused_context) {
+        bus_input_context_commit_text (panel->focused_context, text);
+    }
+}
+
 #define DEFINE_FUNCTION(Name, name)                     \
     void bus_panel_proxy_##name (BusPanelProxy *panel)  \
     {                                                   \
@@ -697,6 +785,10 @@ void
 bus_panel_proxy_focus_in (BusPanelProxy     *panel,
                           BusInputContext   *context)
 {
+    const gchar *path;
+    guint purpose, hints;
+    gint i;
+
     g_assert (BUS_IS_PANEL_PROXY (panel));
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
@@ -709,7 +801,7 @@ bus_panel_proxy_focus_in (BusPanelProxy     *panel,
     g_object_ref_sink (context);
     panel->focused_context = context;
 
-    const gchar *path = ibus_service_get_object_path ((IBusService *)context);
+    path = ibus_service_get_object_path ((IBusService *)context);
 
     g_dbus_proxy_call ((GDBusProxy *)panel,
                        "FocusIn",
@@ -718,7 +810,6 @@ bus_panel_proxy_focus_in (BusPanelProxy     *panel,
                        -1, NULL, NULL, NULL);
 
     /* install signal handlers */
-    gint i;
     for (i = 0; i < G_N_ELEMENTS (input_context_signals); i++) {
         g_signal_connect (context,
                           input_context_signals[i].name,
@@ -726,7 +817,6 @@ bus_panel_proxy_focus_in (BusPanelProxy     *panel,
                           panel);
     }
 
-    guint purpose, hints;
     bus_input_context_get_content_type (context, &purpose, &hints);
     bus_panel_proxy_set_content_type (panel, purpose, hints);
 }
@@ -780,3 +870,10 @@ bus_panel_proxy_destroy_context (BusPanelProxy    *panel,
 
     g_object_unref (context);
 }
+
+PanelType
+bus_panel_proxy_get_panel_type (BusPanelProxy    *panel)
+{
+    g_assert (BUS_IS_PANEL_PROXY (panel));
+    return panel->panel_type;
+}
