@@ -2,25 +2,6 @@
 /* Generated data (by glib-mkenums) */
 
 #include "ibus.h"
-
-/* enumerations from "ibusobject.h" */
-GType
-ibus_object_flags_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GFlagsValue values[] = {
-            { IBUS_IN_DESTRUCTION, "IBUS_IN_DESTRUCTION", "in-destruction" },
-            { IBUS_DESTROYED, "IBUS_DESTROYED", "destroyed" },
-            { IBUS_RESERVED_1, "IBUS_RESERVED_1", "reserved-1" },
-            { IBUS_RESERVED_2, "IBUS_RESERVED_2", "reserved-2" },
-            { 0, NULL, NULL }
-        };
-        etype = g_flags_register_static (g_intern_static_string ("IBusObjectFlags"), values);
-    }
-    return etype;
-}
-
 /* enumerations from "ibusattribute.h" */
 GType
 ibus_attr_type_get_type (void)
@@ -37,7 +18,6 @@ ibus_attr_type_get_type (void)
     }
     return etype;
 }
-
 GType
 ibus_attr_underline_get_type (void)
 {
@@ -55,7 +35,23 @@ ibus_attr_underline_get_type (void)
     }
     return etype;
 }
-
+/* enumerations from "ibusobject.h" */
+GType
+ibus_object_flags_get_type (void)
+{
+    static GType etype = 0;
+    if (G_UNLIKELY(etype == 0)) {
+        static const GFlagsValue values[] = {
+            { IBUS_IN_DESTRUCTION, "IBUS_IN_DESTRUCTION", "in-destruction" },
+            { IBUS_DESTROYED, "IBUS_DESTROYED", "destroyed" },
+            { IBUS_RESERVED_1, "IBUS_RESERVED_1", "reserved-1" },
+            { IBUS_RESERVED_2, "IBUS_RESERVED_2", "reserved-2" },
+            { 0, NULL, NULL }
+        };
+        etype = g_flags_register_static (g_intern_static_string ("IBusObjectFlags"), values);
+    }
+    return etype;
+}
 /* enumerations from "ibusproperty.h" */
 GType
 ibus_prop_type_get_type (void)
@@ -74,7 +70,6 @@ ibus_prop_type_get_type (void)
     }
     return etype;
 }
-
 GType
 ibus_prop_state_get_type (void)
 {
@@ -90,7 +85,6 @@ ibus_prop_state_get_type (void)
     }
     return etype;
 }
-
 /* enumerations from "ibustypes.h" */
 GType
 ibus_modifier_type_get_type (void)
@@ -125,7 +119,6 @@ ibus_modifier_type_get_type (void)
     }
     return etype;
 }
-
 GType
 ibus_capabilite_get_type (void)
 {
@@ -144,7 +137,6 @@ ibus_capabilite_get_type (void)
     }
     return etype;
 }
-
 GType
 ibus_preedit_focus_mode_get_type (void)
 {
@@ -159,7 +151,6 @@ ibus_preedit_focus_mode_get_type (void)
     }
     return etype;
 }
-
 GType
 ibus_orientation_get_type (void)
 {
@@ -175,7 +166,6 @@ ibus_orientation_get_type (void)
     }
     return etype;
 }
-
 GType
 ibus_bus_name_flag_get_type (void)
 {
@@ -191,7 +181,6 @@ ibus_bus_name_flag_get_type (void)
     }
     return etype;
 }
-
 GType
 ibus_bus_request_name_reply_get_type (void)
 {
@@ -208,7 +197,6 @@ ibus_bus_request_name_reply_get_type (void)
     }
     return etype;
 }
-
 GType
 ibus_bus_start_service_by_name_reply_get_type (void)
 {
@@ -223,7 +211,6 @@ ibus_bus_start_service_by_name_reply_get_type (void)
     }
     return etype;
 }
-
 GType
 ibus_error_get_type (void)
 {
@@ -239,7 +226,6 @@ ibus_error_get_type (void)
     }
     return etype;
 }
-
 GType
 ibus_input_purpose_get_type (void)
 {
@@ -262,7 +248,6 @@ ibus_input_purpose_get_type (void)
     }
     return etype;
 }
-
 GType
 ibus_input_hints_get_type (void)
 {
@@ -285,7 +270,24 @@ ibus_input_hints_get_type (void)
     }
     return etype;
 }
-
+/* enumerations from "ibusxevent.h" */
+GType
+ibus_xevent_type_get_type (void)
+{
+    static GType etype = 0;
+    if (G_UNLIKELY(etype == 0)) {
+        static const GEnumValue values[] = {
+            { IBUS_X_EVENT_NOTHING, "IBUS_X_EVENT_NOTHING", "nothing" },
+            { IBUS_X_EVENT_KEY_PRESS, "IBUS_X_EVENT_KEY_PRESS", "key-press" },
+            { IBUS_X_EVENT_KEY_RELEASE, "IBUS_X_EVENT_KEY_RELEASE", "key-release" },
+            { IBUS_X_EVENT_OTHER, "IBUS_X_EVENT_OTHER", "other" },
+            { IBUS_X_EVENT_EVENT_LAST, "IBUS_X_EVENT_EVENT_LAST", "event-last" },
+            { 0, NULL, NULL }
+        };
+        etype = g_enum_register_static (g_intern_static_string ("IBusXEventType"), values);
+    }
+    return etype;
+}
 
 /* Generated data ends here */
 
