@@ -1,7 +1,5 @@
 #include "ibusmarshalers.h"
-
-#include	<glib-object.h>
-
+#include <glib-object.h>
 
 #ifdef G_ENABLE_DEBUG
 #define g_marshal_value_peek_boolean(v)  g_value_get_boolean (v)
@@ -49,14 +47,143 @@
 #define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
 #endif /* !G_ENABLE_DEBUG */
 
-
 /* VOID:NONE (./ibusmarshalers.list:1) */
+void
+_ibus_marshal_VOID__VOID (GClosure     *closure,
+                          GValue       *return_value G_GNUC_UNUSED,
+                          guint         n_param_values,
+                          const GValue *param_values,
+                          gpointer      invocation_hint G_GNUC_UNUSED,
+                          gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__VOID) (gpointer data1,
+                                           gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
+  gpointer data1, data2;
+  GMarshalFunc_VOID__VOID callback;
+
+  g_return_if_fail (n_param_values == 1);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__VOID) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            data2);
+}
 
 /* VOID:OBJECT (./ibusmarshalers.list:2) */
+void
+_ibus_marshal_VOID__OBJECT (GClosure     *closure,
+                            GValue       *return_value G_GNUC_UNUSED,
+                            guint         n_param_values,
+                            const GValue *param_values,
+                            gpointer      invocation_hint G_GNUC_UNUSED,
+                            gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__OBJECT) (gpointer data1,
+                                             gpointer arg1,
+                                             gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
+  gpointer data1, data2;
+  GMarshalFunc_VOID__OBJECT callback;
+
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__OBJECT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_object (param_values + 1),
+            data2);
+}
 
 /* VOID:POINTER (./ibusmarshalers.list:3) */
+void
+_ibus_marshal_VOID__POINTER (GClosure     *closure,
+                             GValue       *return_value G_GNUC_UNUSED,
+                             guint         n_param_values,
+                             const GValue *param_values,
+                             gpointer      invocation_hint G_GNUC_UNUSED,
+                             gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__POINTER) (gpointer data1,
+                                              gpointer arg1,
+                                              gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
+  gpointer data1, data2;
+  GMarshalFunc_VOID__POINTER callback;
+
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__POINTER) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_pointer (param_values + 1),
+            data2);
+}
 
 /* VOID:STRING (./ibusmarshalers.list:4) */
+void
+_ibus_marshal_VOID__STRING (GClosure     *closure,
+                            GValue       *return_value G_GNUC_UNUSED,
+                            guint         n_param_values,
+                            const GValue *param_values,
+                            gpointer      invocation_hint G_GNUC_UNUSED,
+                            gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__STRING) (gpointer data1,
+                                             gpointer arg1,
+                                             gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
+  gpointer data1, data2;
+  GMarshalFunc_VOID__STRING callback;
+
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__STRING) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_string (param_values + 1),
+            data2);
+}
 
 /* VOID:STRING,INT (./ibusmarshalers.list:5) */
 void
@@ -67,13 +194,13 @@ _ibus_marshal_VOID__STRING_INT (GClosure     *closure,
                                 gpointer      invocation_hint G_GNUC_UNUSED,
                                 gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_INT) (gpointer     data1,
-                                                 gpointer     arg_1,
-                                                 gint         arg_2,
-                                                 gpointer     data2);
-  GMarshalFunc_VOID__STRING_INT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__STRING_INT) (gpointer data1,
+                                                 gpointer arg1,
+                                                 gint arg2,
+                                                 gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__STRING_INT callback;
 
   g_return_if_fail (n_param_values == 3);
 
@@ -104,13 +231,13 @@ _ibus_marshal_VOID__STRING_UINT (GClosure     *closure,
                                  gpointer      invocation_hint G_GNUC_UNUSED,
                                  gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_UINT) (gpointer     data1,
-                                                  gpointer     arg_1,
-                                                  guint        arg_2,
-                                                  gpointer     data2);
-  GMarshalFunc_VOID__STRING_UINT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__STRING_UINT) (gpointer data1,
+                                                  gpointer arg1,
+                                                  guint arg2,
+                                                  gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__STRING_UINT callback;
 
   g_return_if_fail (n_param_values == 3);
 
@@ -135,18 +262,18 @@ _ibus_marshal_VOID__STRING_UINT (GClosure     *closure,
 /* BOOL:POINTER (./ibusmarshalers.list:7) */
 void
 _ibus_marshal_BOOLEAN__POINTER (GClosure     *closure,
-                                GValue       *return_value G_GNUC_UNUSED,
+                                GValue       *return_value,
                                 guint         n_param_values,
                                 const GValue *param_values,
                                 gpointer      invocation_hint G_GNUC_UNUSED,
                                 gpointer      marshal_data)
 {
-  typedef gboolean (*GMarshalFunc_BOOLEAN__POINTER) (gpointer     data1,
-                                                     gpointer     arg_1,
-                                                     gpointer     data2);
-  GMarshalFunc_BOOLEAN__POINTER callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef gboolean (*GMarshalFunc_BOOLEAN__POINTER) (gpointer data1,
+                                                     gpointer arg1,
+                                                     gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_BOOLEAN__POINTER callback;
   gboolean v_return;
 
   g_return_if_fail (return_value != NULL);
@@ -174,19 +301,19 @@ _ibus_marshal_BOOLEAN__POINTER (GClosure     *closure,
 /* BOOL:POINTER,POINTER (./ibusmarshalers.list:8) */
 void
 _ibus_marshal_BOOLEAN__POINTER_POINTER (GClosure     *closure,
-                                        GValue       *return_value G_GNUC_UNUSED,
+                                        GValue       *return_value,
                                         guint         n_param_values,
                                         const GValue *param_values,
                                         gpointer      invocation_hint G_GNUC_UNUSED,
                                         gpointer      marshal_data)
 {
-  typedef gboolean (*GMarshalFunc_BOOLEAN__POINTER_POINTER) (gpointer     data1,
-                                                             gpointer     arg_1,
-                                                             gpointer     arg_2,
-                                                             gpointer     data2);
-  GMarshalFunc_BOOLEAN__POINTER_POINTER callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef gboolean (*GMarshalFunc_BOOLEAN__POINTER_POINTER) (gpointer data1,
+                                                             gpointer arg1,
+                                                             gpointer arg2,
+                                                             gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_BOOLEAN__POINTER_POINTER callback;
   gboolean v_return;
 
   g_return_if_fail (return_value != NULL);
@@ -215,19 +342,19 @@ _ibus_marshal_BOOLEAN__POINTER_POINTER (GClosure     *closure,
 /* BOOL:UINT,UINT (./ibusmarshalers.list:9) */
 void
 _ibus_marshal_BOOLEAN__UINT_UINT (GClosure     *closure,
-                                  GValue       *return_value G_GNUC_UNUSED,
+                                  GValue       *return_value,
                                   guint         n_param_values,
                                   const GValue *param_values,
                                   gpointer      invocation_hint G_GNUC_UNUSED,
                                   gpointer      marshal_data)
 {
-  typedef gboolean (*GMarshalFunc_BOOLEAN__UINT_UINT) (gpointer     data1,
-                                                       guint        arg_1,
-                                                       guint        arg_2,
-                                                       gpointer     data2);
-  GMarshalFunc_BOOLEAN__UINT_UINT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef gboolean (*GMarshalFunc_BOOLEAN__UINT_UINT) (gpointer data1,
+                                                       guint arg1,
+                                                       guint arg2,
+                                                       gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_BOOLEAN__UINT_UINT callback;
   gboolean v_return;
 
   g_return_if_fail (return_value != NULL);
@@ -256,20 +383,20 @@ _ibus_marshal_BOOLEAN__UINT_UINT (GClosure     *closure,
 /* BOOL:UINT,UINT,UINT (./ibusmarshalers.list:10) */
 void
 _ibus_marshal_BOOLEAN__UINT_UINT_UINT (GClosure     *closure,
-                                       GValue       *return_value G_GNUC_UNUSED,
+                                       GValue       *return_value,
                                        guint         n_param_values,
                                        const GValue *param_values,
                                        gpointer      invocation_hint G_GNUC_UNUSED,
                                        gpointer      marshal_data)
 {
-  typedef gboolean (*GMarshalFunc_BOOLEAN__UINT_UINT_UINT) (gpointer     data1,
-                                                            guint        arg_1,
-                                                            guint        arg_2,
-                                                            guint        arg_3,
-                                                            gpointer     data2);
-  GMarshalFunc_BOOLEAN__UINT_UINT_UINT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef gboolean (*GMarshalFunc_BOOLEAN__UINT_UINT_UINT) (gpointer data1,
+                                                            guint arg1,
+                                                            guint arg2,
+                                                            guint arg3,
+                                                            gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_BOOLEAN__UINT_UINT_UINT callback;
   gboolean v_return;
 
   g_return_if_fail (return_value != NULL);
@@ -299,18 +426,18 @@ _ibus_marshal_BOOLEAN__UINT_UINT_UINT (GClosure     *closure,
 /* BOOL:ULONG (./ibusmarshalers.list:11) */
 void
 _ibus_marshal_BOOLEAN__ULONG (GClosure     *closure,
-                              GValue       *return_value G_GNUC_UNUSED,
+                              GValue       *return_value,
                               guint         n_param_values,
                               const GValue *param_values,
                               gpointer      invocation_hint G_GNUC_UNUSED,
                               gpointer      marshal_data)
 {
-  typedef gboolean (*GMarshalFunc_BOOLEAN__ULONG) (gpointer     data1,
-                                                   gulong       arg_1,
-                                                   gpointer     data2);
-  GMarshalFunc_BOOLEAN__ULONG callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef gboolean (*GMarshalFunc_BOOLEAN__ULONG) (gpointer data1,
+                                                   gulong arg1,
+                                                   gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_BOOLEAN__ULONG callback;
   gboolean v_return;
 
   g_return_if_fail (return_value != NULL);
@@ -344,15 +471,15 @@ _ibus_marshal_VOID__INT_INT_INT_INT (GClosure     *closure,
                                      gpointer      invocation_hint G_GNUC_UNUSED,
                                      gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__INT_INT_INT_INT) (gpointer     data1,
-                                                      gint         arg_1,
-                                                      gint         arg_2,
-                                                      gint         arg_3,
-                                                      gint         arg_4,
-                                                      gpointer     data2);
-  GMarshalFunc_VOID__INT_INT_INT_INT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__INT_INT_INT_INT) (gpointer data1,
+                                                      gint arg1,
+                                                      gint arg2,
+                                                      gint arg3,
+                                                      gint arg4,
+                                                      gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__INT_INT_INT_INT callback;
 
   g_return_if_fail (n_param_values == 5);
 
@@ -385,13 +512,13 @@ _ibus_marshal_VOID__UINT_UINT (GClosure     *closure,
                                gpointer      invocation_hint G_GNUC_UNUSED,
                                gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__UINT_UINT) (gpointer     data1,
-                                                guint        arg_1,
-                                                guint        arg_2,
-                                                gpointer     data2);
-  GMarshalFunc_VOID__UINT_UINT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__UINT_UINT) (gpointer data1,
+                                                guint arg1,
+                                                guint arg2,
+                                                gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__UINT_UINT callback;
 
   g_return_if_fail (n_param_values == 3);
 
@@ -422,13 +549,13 @@ _ibus_marshal_VOID__INT_UINT (GClosure     *closure,
                               gpointer      invocation_hint G_GNUC_UNUSED,
                               gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__INT_UINT) (gpointer     data1,
-                                               gint         arg_1,
-                                               guint        arg_2,
-                                               gpointer     data2);
-  GMarshalFunc_VOID__INT_UINT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__INT_UINT) (gpointer data1,
+                                               gint arg1,
+                                               guint arg2,
+                                               gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__INT_UINT callback;
 
   g_return_if_fail (n_param_values == 3);
 
@@ -459,14 +586,14 @@ _ibus_marshal_VOID__UINT_UINT_UINT (GClosure     *closure,
                                     gpointer      invocation_hint G_GNUC_UNUSED,
                                     gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__UINT_UINT_UINT) (gpointer     data1,
-                                                     guint        arg_1,
-                                                     guint        arg_2,
-                                                     guint        arg_3,
-                                                     gpointer     data2);
-  GMarshalFunc_VOID__UINT_UINT_UINT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__UINT_UINT_UINT) (gpointer data1,
+                                                     guint arg1,
+                                                     guint arg2,
+                                                     guint arg3,
+                                                     gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__UINT_UINT_UINT callback;
 
   g_return_if_fail (n_param_values == 4);
 
@@ -498,14 +625,14 @@ _ibus_marshal_VOID__OBJECT_UINT_BOOLEAN (GClosure     *closure,
                                          gpointer      invocation_hint G_GNUC_UNUSED,
                                          gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__OBJECT_UINT_BOOLEAN) (gpointer     data1,
-                                                          gpointer     arg_1,
-                                                          guint        arg_2,
-                                                          gboolean     arg_3,
-                                                          gpointer     data2);
-  GMarshalFunc_VOID__OBJECT_UINT_BOOLEAN callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__OBJECT_UINT_BOOLEAN) (gpointer data1,
+                                                          gpointer arg1,
+                                                          guint arg2,
+                                                          gboolean arg3,
+                                                          gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__OBJECT_UINT_BOOLEAN callback;
 
   g_return_if_fail (n_param_values == 4);
 
@@ -537,15 +664,15 @@ _ibus_marshal_VOID__OBJECT_UINT_BOOLEAN_UINT (GClosure     *closure,
                                               gpointer      invocation_hint G_GNUC_UNUSED,
                                               gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__OBJECT_UINT_BOOLEAN_UINT) (gpointer     data1,
-                                                               gpointer     arg_1,
-                                                               guint        arg_2,
-                                                               gboolean     arg_3,
-                                                               guint        arg_4,
-                                                               gpointer     data2);
-  GMarshalFunc_VOID__OBJECT_UINT_BOOLEAN_UINT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__OBJECT_UINT_BOOLEAN_UINT) (gpointer data1,
+                                                               gpointer arg1,
+                                                               guint arg2,
+                                                               gboolean arg3,
+                                                               guint arg4,
+                                                               gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__OBJECT_UINT_BOOLEAN_UINT callback;
 
   g_return_if_fail (n_param_values == 5);
 
@@ -578,14 +705,14 @@ _ibus_marshal_VOID__OBJECT_UINT_UINT (GClosure     *closure,
                                       gpointer      invocation_hint G_GNUC_UNUSED,
                                       gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__OBJECT_UINT_UINT) (gpointer     data1,
-                                                       gpointer     arg_1,
-                                                       guint        arg_2,
-                                                       guint        arg_3,
-                                                       gpointer     data2);
-  GMarshalFunc_VOID__OBJECT_UINT_UINT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__OBJECT_UINT_UINT) (gpointer data1,
+                                                       gpointer arg1,
+                                                       guint arg2,
+                                                       guint arg3,
+                                                       gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__OBJECT_UINT_UINT callback;
 
   g_return_if_fail (n_param_values == 4);
 
@@ -617,13 +744,13 @@ _ibus_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                     gpointer      invocation_hint G_GNUC_UNUSED,
                                     gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__OBJECT_BOOLEAN) (gpointer     data1,
-                                                     gpointer     arg_1,
-                                                     gboolean     arg_2,
-                                                     gpointer     data2);
-  GMarshalFunc_VOID__OBJECT_BOOLEAN callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__OBJECT_BOOLEAN) (gpointer data1,
+                                                     gpointer arg1,
+                                                     gboolean arg2,
+                                                     gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__OBJECT_BOOLEAN callback;
 
   g_return_if_fail (n_param_values == 3);
 
@@ -654,13 +781,13 @@ _ibus_marshal_VOID__BOXED_BOOLEAN (GClosure     *closure,
                                    gpointer      invocation_hint G_GNUC_UNUSED,
                                    gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__BOXED_BOOLEAN) (gpointer     data1,
-                                                    gpointer     arg_1,
-                                                    gboolean     arg_2,
-                                                    gpointer     data2);
-  GMarshalFunc_VOID__BOXED_BOOLEAN callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__BOXED_BOOLEAN) (gpointer data1,
+                                                    gpointer arg1,
+                                                    gboolean arg2,
+                                                    gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__BOXED_BOOLEAN callback;
 
   g_return_if_fail (n_param_values == 3);
 
@@ -683,6 +810,39 @@ _ibus_marshal_VOID__BOXED_BOOLEAN (GClosure     *closure,
 }
 
 /* VOID:BOXED (./ibusmarshalers.list:21) */
+void
+_ibus_marshal_VOID__BOXED (GClosure     *closure,
+                           GValue       *return_value G_GNUC_UNUSED,
+                           guint         n_param_values,
+                           const GValue *param_values,
+                           gpointer      invocation_hint G_GNUC_UNUSED,
+                           gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__BOXED) (gpointer data1,
+                                            gpointer arg1,
+                                            gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
+  gpointer data1, data2;
+  GMarshalFunc_VOID__BOXED callback;
+
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__BOXED) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_boxed (param_values + 1),
+            data2);
+}
 
 /* VOID:STRING,STRING,VARIANT (./ibusmarshalers.list:22) */
 void
@@ -693,14 +853,14 @@ _ibus_marshal_VOID__STRING_STRING_VARIANT (GClosure     *closure,
                                            gpointer      invocation_hint G_GNUC_UNUSED,
                                            gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_STRING_VARIANT) (gpointer     data1,
-                                                            gpointer     arg_1,
-                                                            gpointer     arg_2,
-                                                            gpointer     arg_3,
-                                                            gpointer     data2);
-  GMarshalFunc_VOID__STRING_STRING_VARIANT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__STRING_STRING_VARIANT) (gpointer data1,
+                                                            gpointer arg1,
+                                                            gpointer arg2,
+                                                            gpointer arg3,
+                                                            gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__STRING_STRING_VARIANT callback;
 
   g_return_if_fail (n_param_values == 4);
 
@@ -732,14 +892,14 @@ _ibus_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                           gpointer      invocation_hint G_GNUC_UNUSED,
                                           gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_STRING_STRING) (gpointer     data1,
-                                                           gpointer     arg_1,
-                                                           gpointer     arg_2,
-                                                           gpointer     arg_3,
-                                                           gpointer     data2);
-  GMarshalFunc_VOID__STRING_STRING_STRING callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__STRING_STRING_STRING) (gpointer data1,
+                                                           gpointer arg1,
+                                                           gpointer arg2,
+                                                           gpointer arg3,
+                                                           gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__STRING_STRING_STRING callback;
 
   g_return_if_fail (n_param_values == 4);
 
@@ -763,8 +923,76 @@ _ibus_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
 }
 
 /* VOID:UINT (./ibusmarshalers.list:24) */
+void
+_ibus_marshal_VOID__UINT (GClosure     *closure,
+                          GValue       *return_value G_GNUC_UNUSED,
+                          guint         n_param_values,
+                          const GValue *param_values,
+                          gpointer      invocation_hint G_GNUC_UNUSED,
+                          gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__UINT) (gpointer data1,
+                                           guint arg1,
+                                           gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
+  gpointer data1, data2;
+  GMarshalFunc_VOID__UINT callback;
+
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__UINT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_uint (param_values + 1),
+            data2);
+}
 
 /* VOID:UINT,POINTER (./ibusmarshalers.list:25) */
+void
+_ibus_marshal_VOID__UINT_POINTER (GClosure     *closure,
+                                  GValue       *return_value G_GNUC_UNUSED,
+                                  guint         n_param_values,
+                                  const GValue *param_values,
+                                  gpointer      invocation_hint G_GNUC_UNUSED,
+                                  gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__UINT_POINTER) (gpointer data1,
+                                                   guint arg1,
+                                                   gpointer arg2,
+                                                   gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
+  gpointer data1, data2;
+  GMarshalFunc_VOID__UINT_POINTER callback;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__UINT_POINTER) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_uint (param_values + 1),
+            g_marshal_value_peek_pointer (param_values + 2),
+            data2);
+}
 
 /* VOID:POINTER,UINT (./ibusmarshalers.list:26) */
 void
@@ -775,13 +1003,13 @@ _ibus_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                   gpointer      invocation_hint G_GNUC_UNUSED,
                                   gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__POINTER_UINT) (gpointer     data1,
-                                                   gpointer     arg_1,
-                                                   guint        arg_2,
-                                                   gpointer     data2);
-  GMarshalFunc_VOID__POINTER_UINT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__POINTER_UINT) (gpointer data1,
+                                                   gpointer arg1,
+                                                   guint arg2,
+                                                   gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__POINTER_UINT callback;
 
   g_return_if_fail (n_param_values == 3);
 
@@ -803,21 +1031,56 @@ _ibus_marshal_VOID__POINTER_UINT (GClosure     *closure,
             data2);
 }
 
-/* OBJECT:STRING (./ibusmarshalers.list:27) */
+/* VOID:VARIANT (./ibusmarshalers.list:27) */
+void
+_ibus_marshal_VOID__VARIANT (GClosure     *closure,
+                             GValue       *return_value G_GNUC_UNUSED,
+                             guint         n_param_values,
+                             const GValue *param_values,
+                             gpointer      invocation_hint G_GNUC_UNUSED,
+                             gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__VARIANT) (gpointer data1,
+                                              gpointer arg1,
+                                              gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
+  gpointer data1, data2;
+  GMarshalFunc_VOID__VARIANT callback;
+
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__VARIANT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_variant (param_values + 1),
+            data2);
+}
+
+/* OBJECT:STRING (./ibusmarshalers.list:28) */
 void
 _ibus_marshal_OBJECT__STRING (GClosure     *closure,
-                              GValue       *return_value G_GNUC_UNUSED,
+                              GValue       *return_value,
                               guint         n_param_values,
                               const GValue *param_values,
                               gpointer      invocation_hint G_GNUC_UNUSED,
                               gpointer      marshal_data)
 {
-  typedef GObject* (*GMarshalFunc_OBJECT__STRING) (gpointer     data1,
-                                                   gpointer     arg_1,
-                                                   gpointer     data2);
-  GMarshalFunc_OBJECT__STRING callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef GObject* (*GMarshalFunc_OBJECT__STRING) (gpointer data1,
+                                                   gpointer arg1,
+                                                   gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_OBJECT__STRING callback;
   GObject* v_return;
 
   g_return_if_fail (return_value != NULL);
